@@ -46,7 +46,7 @@ vtkInformationKeyMacro(vtkDemandDrivenPipeline, REQUEST_DATA_NOT_GENERATED, Requ
 vtkInformationKeyMacro(vtkDemandDrivenPipeline, REQUEST_DATA_OBJECT, Request);
 vtkInformationKeyMacro(vtkDemandDrivenPipeline, REQUEST_INFORMATION, Request);
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkDemandDrivenPipeline::vtkDemandDrivenPipeline()
 {
   this->InfoRequest = nullptr;
@@ -55,7 +55,7 @@ vtkDemandDrivenPipeline::vtkDemandDrivenPipeline()
   this->PipelineMTime = 0;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkDemandDrivenPipeline::~vtkDemandDrivenPipeline()
 {
   if (this->InfoRequest)
@@ -72,14 +72,14 @@ vtkDemandDrivenPipeline::~vtkDemandDrivenPipeline()
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkDemandDrivenPipeline::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os, indent);
   os << indent << "PipelineMTime: " << this->PipelineMTime << "\n";
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::ComputePipelineMTime(vtkInformation* request,
   vtkInformationVector** inInfoVec, vtkInformationVector* outInfoVec, int requestFromOutputPort,
   vtkMTimeType* mtime)
@@ -96,17 +96,16 @@ int vtkDemandDrivenPipeline::ComputePipelineMTime(vtkInformation* request,
   {
     if (request)
     {
-      vtkErrorMacro("Algorithm " << this->Algorithm->GetClassName() << "(" << this->Algorithm
-                                 << ") returned failure for pipeline"
+      vtkErrorMacro("Algorithm " << this->Algorithm->GetObjectDescription()
+                                 << " returned failure for pipeline"
                                  << " modified time request from output port "
                                  << requestFromOutputPort << ": " << *request);
     }
     else
     {
-      vtkErrorMacro("Algorithm " << this->Algorithm->GetClassName() << "(" << this->Algorithm
-                                 << ") returned failure for pipeline"
-                                 << " modified time request from output port "
-                                 << requestFromOutputPort << ".");
+      vtkErrorMacro(
+        "Algorithm " << this->Algorithm->GetObjectDescription() << " returned failure for pipeline"
+                     << " modified time request from output port " << requestFromOutputPort << ".");
     }
     return 0;
   }
@@ -144,7 +143,7 @@ int vtkDemandDrivenPipeline::ComputePipelineMTime(vtkInformation* request,
   return 1;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkTypeBool vtkDemandDrivenPipeline::ProcessRequest(
   vtkInformation* request, vtkInformationVector** inInfoVec, vtkInformationVector* outInfoVec)
 {
@@ -281,16 +280,16 @@ vtkTypeBool vtkDemandDrivenPipeline::ProcessRequest(
   return this->Superclass::ProcessRequest(request, inInfoVec, outInfoVec);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkDemandDrivenPipeline::ResetPipelineInformation(int, vtkInformation*) {}
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkTypeBool vtkDemandDrivenPipeline::Update()
 {
   return this->Superclass::Update();
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkTypeBool vtkDemandDrivenPipeline::Update(int port)
 {
   if (!this->UpdateInformation())
@@ -307,7 +306,7 @@ vtkTypeBool vtkDemandDrivenPipeline::Update(int port)
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::UpdatePipelineMTime()
 {
   // The algorithm should not invoke anything on the executive.
@@ -323,7 +322,7 @@ int vtkDemandDrivenPipeline::UpdatePipelineMTime()
   return 1;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::UpdateDataObject()
 {
   // The algorithm should not invoke anything on the executive.
@@ -354,7 +353,7 @@ int vtkDemandDrivenPipeline::UpdateDataObject()
     this->DataObjectRequest, this->GetInputInformation(), this->GetOutputInformation());
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::UpdateInformation()
 {
   // The algorithm should not invoke anything on the executive.
@@ -385,7 +384,7 @@ int vtkDemandDrivenPipeline::UpdateInformation()
     this->InfoRequest, this->GetInputInformation(), this->GetOutputInformation());
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::UpdateData(int outputPort)
 {
   // The algorithm should not invoke anything on the executive.
@@ -420,7 +419,7 @@ int vtkDemandDrivenPipeline::UpdateData(int outputPort)
     this->DataRequest, this->GetInputInformation(), this->GetOutputInformation());
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::ExecuteDataObject(
   vtkInformation* request, vtkInformationVector** inInfo, vtkInformationVector* outInfo)
 {
@@ -436,7 +435,7 @@ int vtkDemandDrivenPipeline::ExecuteDataObject(
   return result;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::ExecuteInformation(
   vtkInformation* request, vtkInformationVector** inInfoVec, vtkInformationVector* outInfoVec)
 {
@@ -452,7 +451,7 @@ int vtkDemandDrivenPipeline::ExecuteInformation(
   return this->CallAlgorithm(request, vtkExecutive::RequestDownstream, inInfoVec, outInfoVec);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::ExecuteData(
   vtkInformation* request, vtkInformationVector** inInfo, vtkInformationVector* outInfo)
 {
@@ -472,7 +471,7 @@ int vtkDemandDrivenPipeline::ExecuteData(
   return result;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkDemandDrivenPipeline::ExecuteDataStart(
   vtkInformation* request, vtkInformationVector** inInfo, vtkInformationVector* outputs)
 {
@@ -524,15 +523,11 @@ void vtkDemandDrivenPipeline::ExecuteDataStart(
   this->Algorithm->UpdateProgress(0.0);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkDemandDrivenPipeline::ExecuteDataEnd(
   vtkInformation* request, vtkInformationVector** inInfoVec, vtkInformationVector* outputs)
 {
-  // The algorithm has either finished or aborted.
-  if (!this->Algorithm->GetAbortExecute())
-  {
-    this->Algorithm->UpdateProgress(1.0);
-  }
+  this->Algorithm->UpdateProgress(1.0);
 
   // Tell observers the algorithm is done executing.
   this->Algorithm->InvokeEvent(vtkCommand::EndEvent, nullptr);
@@ -555,7 +550,7 @@ void vtkDemandDrivenPipeline::ExecuteDataEnd(
     {
       vtkInformation* inInfo = inInfoVec[i]->GetInformationObject(j);
       vtkDataObject* dataObject = inInfo->Get(vtkDataObject::DATA_OBJECT());
-      if (dataObject && (dataObject->GetGlobalReleaseDataFlag() || inInfo->Get(RELEASE_DATA())))
+      if (dataObject && (vtkDataObject::GetGlobalReleaseDataFlag() || inInfo->Get(RELEASE_DATA())))
       {
         dataObject->ReleaseData();
       }
@@ -563,7 +558,7 @@ void vtkDemandDrivenPipeline::ExecuteDataEnd(
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkDemandDrivenPipeline::MarkOutputsGenerated(
   vtkInformation*, vtkInformationVector** /* inInfoVec */, vtkInformationVector* outputs)
 {
@@ -579,7 +574,7 @@ void vtkDemandDrivenPipeline::MarkOutputsGenerated(
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::CheckDataObject(int port, vtkInformationVector* outInfoVec)
 {
   // Check that the given output port has a valid data object.
@@ -609,8 +604,8 @@ int vtkDemandDrivenPipeline::CheckDataObject(int port, vtkInformationVector* out
     if (!data)
     {
       // The algorithm has a bug and did not create the data object.
-      vtkErrorMacro("Algorithm " << this->Algorithm->GetClassName() << "(" << this->Algorithm
-                                 << ") did not create output for port " << port
+      vtkErrorMacro("Algorithm " << this->Algorithm->GetObjectDescription()
+                                 << " did not create output for port " << port
                                  << " when asked by REQUEST_DATA_OBJECT and does not"
                                  << " specify a concrete DATA_TYPE_NAME.");
       return 0;
@@ -627,15 +622,15 @@ int vtkDemandDrivenPipeline::CheckDataObject(int port, vtkInformationVector* out
   {
     // The algorithm did not specify its output data type and no
     // object exists.
-    vtkErrorMacro("Algorithm " << this->Algorithm->GetClassName() << "(" << this->Algorithm
-                               << ") did not create output for port " << port
+    vtkErrorMacro("Algorithm " << this->Algorithm->GetObjectDescription()
+                               << " did not create output for port " << port
                                << " when asked by REQUEST_DATA_OBJECT and does not"
                                << " specify any DATA_TYPE_NAME.");
     return 0;
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::InputCountIsValid(vtkInformationVector** inInfoVec)
 {
   // Check the number of connections for each port.
@@ -650,7 +645,7 @@ int vtkDemandDrivenPipeline::InputCountIsValid(vtkInformationVector** inInfoVec)
   return result;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::InputCountIsValid(int port, vtkInformationVector** inInfoVec)
 {
   // Get the number of connections for this port.
@@ -663,8 +658,8 @@ int vtkDemandDrivenPipeline::InputCountIsValid(int port, vtkInformationVector**
   // If the input port is optional, there may be less than one connection.
   if (!this->InputIsOptional(port) && (connections < 1))
   {
-    vtkErrorMacro("Input port " << port << " of algorithm " << this->Algorithm->GetClassName()
-                                << "(" << this->Algorithm << ") has " << connections
+    vtkErrorMacro("Input port " << port << " of algorithm "
+                                << this->Algorithm->GetObjectDescription() << " has " << connections
                                 << " connections but is not optional.");
     return 0;
   }
@@ -672,15 +667,15 @@ int vtkDemandDrivenPipeline::InputCountIsValid(int port, vtkInformationVector**
   // If the input port is repeatable, there may be more than one connection.
   if (!this->InputIsRepeatable(port) && (connections > 1))
   {
-    vtkErrorMacro("Input port " << port << " of algorithm " << this->Algorithm->GetClassName()
-                                << "(" << this->Algorithm << ") has " << connections
+    vtkErrorMacro("Input port " << port << " of algorithm "
+                                << this->Algorithm->GetObjectDescription() << " has " << connections
                                 << " connections but is not repeatable.");
     return 0;
   }
   return 1;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::InputTypeIsValid(vtkInformationVector** inInfoVec)
 {
   // Check the connection types for each port.
@@ -695,7 +690,7 @@ int vtkDemandDrivenPipeline::InputTypeIsValid(vtkInformationVector** inInfoVec)
   return result;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::InputTypeIsValid(int port, vtkInformationVector** inInfoVec)
 {
   // Check the type of each connection on this port.
@@ -714,7 +709,7 @@ int vtkDemandDrivenPipeline::InputTypeIsValid(int port, vtkInformationVector** i
   return result;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::InputTypeIsValid(int port, int index, vtkInformationVector** inInfoVec)
 {
   if (!inInfoVec[port])
@@ -733,7 +728,7 @@ int vtkDemandDrivenPipeline::InputTypeIsValid(int port, int index, vtkInformatio
     {
       vtkErrorMacro("Input for connection index "
         << index << " on input port index " << port << " for algorithm "
-        << this->Algorithm->GetClassName() << "(" << this->Algorithm << ") is nullptr, but a "
+        << this->Algorithm->GetObjectDescription() << " is nullptr, but a "
         << info->Get(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), 0) << " is required.");
       return 0;
     }
@@ -755,16 +750,15 @@ int vtkDemandDrivenPipeline::InputTypeIsValid(int port, int index, vtkInformatio
     {
       vtkErrorMacro("Input for connection index "
         << index << " on input port index " << port << " for algorithm "
-        << this->Algorithm->GetClassName() << "(" << this->Algorithm << ") is of type "
-        << input->GetClassName() << ", but a "
-        << info->Get(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), 0) << " is required.");
+        << this->Algorithm->GetObjectDescription() << " is of type " << input->GetClassName()
+        << ", but a " << info->Get(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), 0) << " is required.");
       return 0;
     }
   }
   return 1;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::InputFieldsAreValid(vtkInformationVector** inInfoVec)
 {
   // Check the fields for each port.
@@ -779,7 +773,7 @@ int vtkDemandDrivenPipeline::InputFieldsAreValid(vtkInformationVector** inInfoVe
   return result;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::InputFieldsAreValid(int port, vtkInformationVector** inInfoVec)
 {
   // Check the fields for each connection on this port.
@@ -798,7 +792,7 @@ int vtkDemandDrivenPipeline::InputFieldsAreValid(int port, vtkInformationVector*
   return result;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::InputFieldsAreValid(
   int port, int index, vtkInformationVector** inInfoVec)
 {
@@ -869,7 +863,7 @@ int vtkDemandDrivenPipeline::InputFieldsAreValid(
   return result;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::DataSetAttributeExists(
   vtkDataSetAttributes* dsa, vtkInformation* field)
 {
@@ -886,7 +880,7 @@ int vtkDemandDrivenPipeline::DataSetAttributeExists(
   }
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::FieldArrayExists(vtkFieldData* data, vtkInformation* field)
 {
   // Search the field data instance for an array matching the requirements.
@@ -901,7 +895,7 @@ int vtkDemandDrivenPipeline::FieldArrayExists(vtkFieldData* data, vtkInformation
   return 0;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::ArrayIsValid(vtkAbstractArray* array, vtkInformation* field)
 {
   // Enforce existence of the array.
@@ -954,7 +948,7 @@ int vtkDemandDrivenPipeline::ArrayIsValid(vtkAbstractArray* array, vtkInformatio
   return 1;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::InputIsOptional(int port)
 {
   if (vtkInformation* info = this->Algorithm->GetInputPortInformation(port))
@@ -964,7 +958,7 @@ int vtkDemandDrivenPipeline::InputIsOptional(int port)
   return 0;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::InputIsRepeatable(int port)
 {
   if (vtkInformation* info = this->Algorithm->GetInputPortInformation(port))
@@ -974,13 +968,13 @@ int vtkDemandDrivenPipeline::InputIsRepeatable(int port)
   return 0;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkDataObject* vtkDemandDrivenPipeline::NewDataObject(const char* type)
 {
   return vtkDataObjectTypes::NewDataObject(type);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline ::NeedToExecuteData(
   int outputPort, vtkInformationVector** inInfoVec, vtkInformationVector* outInfoVec)
 {
@@ -1020,7 +1014,7 @@ int vtkDemandDrivenPipeline ::NeedToExecuteData(
   return 0;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::SetReleaseDataFlag(int port, int n)
 {
   if (!this->OutputPortIndexInRange(port, "set release data flag on"))
@@ -1036,7 +1030,7 @@ int vtkDemandDrivenPipeline::SetReleaseDataFlag(int port, int n)
   return 0;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkDemandDrivenPipeline::GetReleaseDataFlag(int port)
 {
   if (!this->OutputPortIndexInRange(port, "get release data flag from"))
