@@ -173,7 +173,7 @@ bool vtkMatplotlibMathTextUtilities::InitializeMaskParser()
     return false;
   }
 
-  this->MaskParser = PyObject_CallFunction(mathTextParserClass, const_cast<char*>("s"), "bitmap");
+  this->MaskParser = PyObject_CallFunction(mathTextParserClass, const_cast<char*>("s"), "agg");
   if (this->CheckForError(this->MaskParser))
   {
     Py_CLEAR(this->MaskParser);
@@ -682,7 +682,7 @@ bool vtkMatplotlibMathTextUtilities::ComputeCellRowsAndCols(const char* str, PyO
   }
 
   // Get ftimage
-  PyObject* ftImage = PyTuple_GetItem(resTupleParse, 0);
+  PyObject* ftImage = PyTuple_GetItem(resTupleParse, 5);
   if (this->CheckForError(ftImage))
   {
     return false;
@@ -788,6 +788,7 @@ void vtkMatplotlibMathTextUtilities::ComputeTextColors(vtkTextProperty* tprop, T
 //------------------------------------------------------------------------------
 bool vtkMatplotlibMathTextUtilities::SetMathTextFont(vtkTextProperty* tprop)
 {
+  vtkPythonScopeGilEnsurer gilEnsurer;
   vtkSmartPyObject mplBase(PyImport_ImportModule("matplotlib"));
   if (this->CheckForError(mplBase))
   {
