@@ -8,24 +8,27 @@ readonly name="hdf5"
 readonly ownership="HDF Upstream <kwrobot@kitware.com>"
 readonly subtree="ThirdParty/$name/vtk$name"
 readonly repo="https://gitlab.kitware.com/third-party/hdf5.git"
-readonly tag="for/vtk-20210227-1.10.6"
+readonly tag="for/vtk-20220414-1.13.1"
 readonly paths="
 CMakeFilters.cmake
 CMakeInstallation.cmake
 CMakeLists.txt
+CMakePlugins.cmake
 UserMacros.cmake
 
 src/
 hl/CMakeLists.txt
-hl/COPYING
 hl/src/
 
 config/lt_vers.am
-config/COPYING
+config/clang-warnings
+config/gnu-warnings
+config/intel-warnings
 config/cmake/ConfigureChecks.cmake
 config/cmake/ConversionTests.c
 config/cmake/H5pubconf.h.in
 config/cmake/HDF5Macros.cmake
+config/cmake/HDF5PluginMacros.cmake
 config/cmake/HDFCompilerFlags.cmake
 config/cmake/libhdf5.settings.cmake.in
 
@@ -38,7 +41,6 @@ config/cmake_ext_mod/HDFTests.c
 ACKNOWLEDGMENTS
 COPYING
 COPYING_LBNL_HDF5
-README.txt
 README.kitware.md
 "
 
@@ -49,6 +51,8 @@ extract_source () {
     find . -name Makefile.in -delete
     find . -name "*.lnt" -delete
     rm -v src/.indent.pro
+    # Add missing newline at EOF.
+    echo >> config/intel-warnings/general
     popd
 }
 
