@@ -8,7 +8,7 @@ readonly name="sqlite"
 readonly ownership="sqlite Upstream <kwrobot@kitware.com>"
 readonly subtree="ThirdParty/$name/vtk$name"
 readonly repo="https://gitlab.kitware.com/third-party/sqlite.git"
-readonly tag="for/vtk-old"
+readonly tag="for/vtk-20210924-3.36.0" # see `manifest` below if the version number changes
 readonly paths="
 .gitattributes
 CMakeLists.txt
@@ -26,12 +26,12 @@ main.mk
 extract_source () {
     git_archive
     pushd "$extractdir/$name-reduced"
-    echo "3.21.0-vtk" > manifest
-    echo "3.21.0-vtk" > manifest.uuid
+    echo "3.36.0-vtk" > manifest
+    echo "3.36.0-vtk" > manifest.uuid
     make -f main.mk TOP=$PWD BCC=cc target_source sqlite3.c
     rm -rvf ext src tool main.mk manifest manifest.uuid VERSION
     rm -rvf lemon keywordhash.h lempar.c mkkeywordhash mksourceid
-    rm -rvf opcodes.* parse.* shell.c tsrc fts5.* fts5parse.*
+    rm -rvf opcodes.* parse.* tsrc fts5.* fts5parse.*
     rm -rvf sqlite3ext.h sqlite3session.h tclsqlite3.c target_source
     popd
 }
